@@ -51,7 +51,6 @@ typedef enum {
 typedef struct {
   guint64 fd;
   afc_client_t afc_cli;
-  char *app;
 } FileHandle;
 
 typedef struct {
@@ -59,7 +58,6 @@ typedef struct {
   char *id;
   char *icon_path;
   house_arrest_client_t house_arrest;
-  guint num_users;
   afc_client_t afc_cli;
 } AppInfo;
 
@@ -591,22 +589,7 @@ g_vfs_backend_afc_mount (GVfsBackend *backend,
         continue;
       }
 
-    if (lerr == LOCKDOWN_E_USER_DENIED_PAIRING)
-      {
-        aborted = TRUE;
-        break;
-      }
-
-    /* An unknown error? Let's try again without prompting */
-    if (lerr == LOCKDOWN_E_UNKNOWN_ERROR)
-      {
-        g_debug ("Got an unknown lockdown error, retrying after a short sleep\n");
-        g_usleep (G_USEC_PER_SEC);
-        continue;
-      }
-
-    if (lerr != LOCKDOWN_E_PASSWORD_PROTECTED &&
-        lerr != LOCKDOWN_E_PAIRING_DIALOG_RESPONSE_PENDING)
+    if (lerr != LOCKDOWN_E_PASSWORD_PROTECTED)
       break;
 
     aborted = FALSE;
@@ -778,9 +761,8 @@ is_regular (GVfsBackendAfc *backend,
   return result;
 }
 
-static gboolean
+static void
 g_vfs_backend_setup_afc_for_app (GVfsBackendAfc *self,
-                                 gboolean        last_try,
                                  const char     *id)
 {
   AppInfo *info;
@@ -790,36 +772,27 @@ g_vfs_backend_setup_afc_for_app (GVfsBackendAfc *self,
   afc_client_t afc;
   plist_t dict, error;
   lockdownd_error_t lerr;
-  gboolean retry = FALSE;
-
-  g_mutex_lock (&self->apps_lock);
 
   info = g_hash_table_lookup (self->apps, id);
 
   if (info == NULL ||
       info->afc_cli != NULL)
-    goto out;
+    return;
 
   /* Load house arrest and afc now! */
   lockdown_cli = NULL;
   if (lockdownd_client_new_with_handshake (self->dev, &lockdown_cli, "gvfsd-afc") != LOCKDOWN_E_SUCCESS)
     {
       g_warning ("Failed to get a lockdown to start house arrest for app %s", info->id);
-      goto out;
+      return;
     }
 
   lerr = lockdownd_start_service (lockdown_cli, "com.apple.mobile.house_arrest", &lockdown_service);
   if (lerr != LOCKDOWN_E_SUCCESS)
     {
-      if (lerr == LOCKDOWN_E_SERVICE_LIMIT && !last_try)
-        {
-          retry = TRUE;
-          g_debug ("Failed to start house arrest for app %s (%d)\n", info->id, lerr);
-        }
-      else
-        g_warning ("Failed to start house arrest for app %s (%d)", info->id, lerr);
       lockdownd_client_free (lockdown_cli);
-      goto out;
+      g_warning ("Failed to start house arrest for app %s", info->id);
+      return;
     }
 
   house_arrest = NULL;
@@ -829,7 +802,7 @@ g_vfs_backend_setup_afc_for_app (GVfsBackendAfc *self,
       g_warning ("Failed to start house arrest client for app %s", info->id);
       lockdownd_client_free (lockdown_cli);
       lockdownd_service_descriptor_free (lockdown_service);
-      goto out;
+      return;
     }
 
   lockdownd_service_descriptor_free (lockdown_service);
@@ -841,7 +814,7 @@ g_vfs_backend_setup_afc_for_app (GVfsBackendAfc *self,
       g_warning ("Failed to set up house arrest for app %s", info->id);
       house_arrest_client_free (house_arrest);
       lockdownd_client_free (lockdown_cli);
-      goto out;
+      return;
     }
   error = plist_dict_get_item (dict, "Error");
   if (error != NULL)
@@ -855,7 +828,7 @@ g_vfs_backend_setup_afc_for_app (GVfsBackendAfc *self,
 
       house_arrest_client_free (house_arrest);
       lockdownd_client_free (lockdown_cli);
-      goto out;
+      return;
     }
   plist_free (dict);
 
@@ -867,109 +840,12 @@ g_vfs_backend_setup_afc_for_app (GVfsBackendAfc *self,
     {
       g_warning ("Failed to set up afc client for app %s", info->id);
       house_arrest_client_free (house_arrest);
-      goto out;
+
+      return;
     }
 
   info->house_arrest = house_arrest;
   info->afc_cli = afc;
-
-out:
-  g_mutex_unlock (&self->apps_lock);
-  return !retry;
-}
-
-static FileHandle *
-g_vfs_backend_file_handle_new (GVfsBackendAfc *self,
-                               const char     *app)
-{
-  AppInfo *info;
-  FileHandle *handle;
-
-  handle = g_new0 (FileHandle, 1);
-
-  if (app == NULL)
-    return handle;
-
-  g_mutex_lock (&self->apps_lock);
-  info = g_hash_table_lookup (self->apps, app);
-  info->num_users++;
-  g_mutex_unlock (&self->apps_lock);
-
-  handle->app = g_strdup (app);
-
-  return handle;
-}
-
-static void
-g_vfs_backend_file_handle_free (GVfsBackendAfc *self,
-                                FileHandle     *fh)
-{
-  AppInfo *info;
-
-  if (fh == NULL)
-    return;
-
-  if (fh->app == NULL)
-    goto out;
-
-  g_mutex_lock (&self->apps_lock);
-  info = g_hash_table_lookup (self->apps, fh->app);
-  g_assert (info->num_users != 0);
-  info->num_users--;
-  g_mutex_unlock (&self->apps_lock);
-
-out:
-  if (self->connected)
-    afc_file_close (fh->afc_cli, fh->fd);
-  g_free (fh->app);
-  g_free (fh);
-}
-
-/* If we succeeded in removing access to at least one
- * HouseArrest service, return TRUE */
-static gboolean
-g_vfs_backend_gc_house_arrest (GVfsBackendAfc *self,
-                               const char     *app)
-{
-  GList *apps, *l;
-  gboolean ret = FALSE;
-
-  g_mutex_lock (&self->apps_lock);
-
-  apps = g_hash_table_get_values (self->apps);
-  /* XXX: We might want to sort the apps so the
-   * oldest used gets cleaned up first */
-
-  for (l = apps; l != NULL; l = l->next)
-    {
-      AppInfo *info = l->data;
-
-      /* Don't close the same app we're trying to
-       * connect to the service, but return as it's
-       * already setup */
-      if (g_strcmp0 (info->id, app) == 0)
-        {
-          g_debug ("A HouseArrest service for '%s' is already setup\n", app);
-          ret = TRUE;
-          break;
-        }
-
-      if (info->afc_cli == NULL ||
-          info->num_users > 0)
-        continue;
-
-      g_clear_pointer (&info->afc_cli, afc_client_free);
-      g_clear_pointer (&info->house_arrest, house_arrest_client_free);
-
-      g_debug ("Managed to free HouseArrest service from '%s', for '%s'\n",
-               info->id, app);
-      ret = TRUE;
-      break;
-    }
-
-  g_mutex_unlock (&self->apps_lock);
-
-  return ret;
 }
 
 /* If force_afc_mount is TRUE, then we'll try to mount
@@ -978,12 +854,12 @@ static char *
 g_vfs_backend_parse_house_arrest_path (GVfsBackendAfc *self,
                                        gboolean        force_afc_mount,
                                        const char     *path,
-                                       gboolean       *is_doc_root_ret,
                                        char          **new_path)
 {
   char **comps;
+  char *s;
   char *app;
-  gboolean setup_afc, is_doc_root;
+  gboolean setup_afc;
 
   if (path == NULL || *path == '\0' || g_str_equal (path, "/"))
     {
@@ -998,28 +874,28 @@ g_vfs_backend_parse_house_arrest_path (GVfsBackendAfc *self,
 
   setup_afc = force_afc_mount;
   app = comps[0];
-  is_doc_root = (comps[0] != NULL && comps[1] == NULL);
-  if (is_doc_root_ret)
-    *is_doc_root_ret = is_doc_root;
 
   /* Replace the app path with "Documents" so the gvfs
    * path of afc://<uuid>/org.gnome.test/foo.txt should
    * correspond to Documents/foo.txt in the app's container */
   comps[0] = g_strdup ("Documents");
-  *new_path = g_strjoinv ("/", comps);
-  if (is_doc_root)
-    setup_afc = TRUE;
+  s = g_strjoinv ("/", comps);
+  if (*s == '\0')
+    {
+      g_free (s);
+      *new_path = g_strdup ("/");
+    }
+  else
+    {
+      *new_path = s;
+      setup_afc = TRUE;
+    }
   g_strfreev (comps);
 
   if (app != NULL &&
       setup_afc)
     {
-      if (!g_vfs_backend_setup_afc_for_app (self, FALSE, app))
-        {
-          g_debug ("Ran out of HouseArrest clients for app '%s', trying again\n", app);
-          g_vfs_backend_gc_house_arrest (self, app);
-          g_vfs_backend_setup_afc_for_app (self, TRUE, app);
-        }
+      g_vfs_backend_setup_afc_for_app (self, app);
     }
 
   return app;
@@ -1036,19 +912,21 @@ g_vfs_backend_afc_open_for_read (GVfsBackend *backend,
   char *new_path;
   afc_client_t afc_cli;
   FileHandle *handle;
-  char *app = NULL;
 
   self = G_VFS_BACKEND_AFC(backend);
   g_return_if_fail (self->connected);
 
   if (self->mode == ACCESS_MODE_HOUSE_ARREST)
     {
+      char *app;
       AppInfo *info;
-      gboolean is_doc_root;
 
       new_path = NULL;
-      app = g_vfs_backend_parse_house_arrest_path (self, FALSE, path, &is_doc_root, &new_path);
-      if (app == NULL || is_doc_root)
+      app = g_vfs_backend_parse_house_arrest_path (self, FALSE, path, &new_path);
+      if (app == NULL)
+        goto is_dir_bail;
+
+      if (g_str_equal (new_path, "/"))
         goto is_dir_bail;
 
       info = g_hash_table_lookup (self->apps, app);
@@ -1056,6 +934,7 @@ g_vfs_backend_afc_open_for_read (GVfsBackend *backend,
         goto not_found_bail;
 
       afc_cli = info->afc_cli;
+      g_free (app);
     }
   else
     {
@@ -1067,7 +946,6 @@ g_vfs_backend_afc_open_for_read (GVfsBackend *backend,
     {
 is_dir_bail:
       g_free (new_path);
-      g_free (app);
       g_vfs_job_failed (G_VFS_JOB (job), G_IO_ERROR,
                         G_IO_ERROR_IS_DIRECTORY,
                         _("Can't open directory"));
@@ -1078,7 +956,6 @@ is_dir_bail:
     {
 not_found_bail:
       g_free (new_path);
-      g_free (app);
       g_vfs_job_failed (G_VFS_JOB (job), G_IO_ERROR,
                         G_IO_ERROR_NOT_FOUND,
                         _("File doesn't exist"));
@@ -1089,16 +966,13 @@ not_found_bail:
                                                          new_path ? new_path : path, AFC_FOPEN_RDONLY, &fd),
                                           G_VFS_JOB(job))))
     {
-      g_free (app);
       return;
     }
 
-  handle = g_vfs_backend_file_handle_new (self, app);
+  handle = g_new0 (FileHandle, 1);
   handle->fd = fd;
   handle->afc_cli = afc_cli;
 
-  g_free (app);
-
   g_vfs_job_open_for_read_set_handle (job, handle);
   g_vfs_job_open_for_read_set_can_seek (job, TRUE);
   g_vfs_job_succeeded (G_VFS_JOB(job));
@@ -1115,7 +989,7 @@ g_vfs_backend_afc_create (GVfsBackend *backend,
 {
   uint64_t fd = 0;
   GVfsBackendAfc *self;
-  char *new_path, *app = NULL;
+  char *new_path, *app;
   afc_client_t afc_cli;
   FileHandle *fh;
 
@@ -1128,7 +1002,7 @@ g_vfs_backend_afc_create (GVfsBackend *backend,
     {
       AppInfo *info;
 
-      app = g_vfs_backend_parse_house_arrest_path (self, FALSE, path, NULL, &new_path);
+      app = g_vfs_backend_parse_house_arrest_path (self, FALSE, path, &new_path);
       if (app == NULL)
         {
           g_vfs_backend_afc_check (AFC_E_PERM_DENIED, G_VFS_JOB(job));
@@ -1137,11 +1011,11 @@ g_vfs_backend_afc_create (GVfsBackend *backend,
       info = g_hash_table_lookup (self->apps, app);
       if (info == NULL)
         {
-          g_free (app);
           g_vfs_backend_afc_check (AFC_E_OBJECT_NOT_FOUND, G_VFS_JOB(job));
           return;
         }
       afc_cli = info->afc_cli;
+      g_free (app);
     }
   else
     {
@@ -1154,18 +1028,15 @@ g_vfs_backend_afc_create (GVfsBackend *backend,
                                           G_VFS_JOB(job))))
     {
       g_free (new_path);
-      g_free (app);
       return;
     }
 
   g_free (new_path);
 
-  fh = g_vfs_backend_file_handle_new (self, app);
+  fh = g_new0 (FileHandle, 1);
   fh->fd = fd;
   fh->afc_cli = afc_cli;
 
-  g_free (app);
-
   g_vfs_job_open_for_write_set_handle (job, fh);
   g_vfs_job_open_for_write_set_can_seek (job, TRUE);
   g_vfs_job_open_for_write_set_can_truncate (job, TRUE);
@@ -1184,7 +1055,7 @@ g_vfs_backend_afc_append_to (GVfsBackend *backend,
   uint64_t fd = 0;
   uint64_t off = 0;
   GVfsBackendAfc *self;
-  char *new_path, *app = NULL;
+  char *new_path, *app;
   afc_client_t afc_cli;
   FileHandle *fh;
 
@@ -1197,7 +1068,7 @@ g_vfs_backend_afc_append_to (GVfsBackend *backend,
     {
       AppInfo *info;
 
-      app = g_vfs_backend_parse_house_arrest_path (self, FALSE, path, NULL, &new_path);
+      app = g_vfs_backend_parse_house_arrest_path (self, FALSE, path, &new_path);
       if (app == NULL)
         {
           g_vfs_backend_afc_check (AFC_E_PERM_DENIED, G_VFS_JOB(job));
@@ -1206,11 +1077,11 @@ g_vfs_backend_afc_append_to (GVfsBackend *backend,
       info = g_hash_table_lookup (self->apps, app);
       if (info == NULL)
         {
-          g_free (app);
           g_vfs_backend_afc_check (AFC_E_OBJECT_NOT_FOUND, G_VFS_JOB(job));
           return;
         }
       afc_cli = info->afc_cli;
+      g_free (app);
     }
   else
     {
@@ -1223,7 +1094,6 @@ g_vfs_backend_afc_append_to (GVfsBackend *backend,
                                           G_VFS_JOB(job))))
     {
       g_free (new_path);
-      g_free (app);
       return;
     }
 
@@ -1234,7 +1104,6 @@ g_vfs_backend_afc_append_to (GVfsBackend *backend,
                                           G_VFS_JOB(job))))
     {
       afc_file_close (afc_cli, fd);
-      g_free (app);
       return;
     }
 
@@ -1243,16 +1112,13 @@ g_vfs_backend_afc_append_to (GVfsBackend *backend,
                                           G_VFS_JOB(job))))
     {
       afc_file_close (afc_cli, fd);
-      g_free (app);
       return;
     }
 
-  fh = g_vfs_backend_file_handle_new (self, app);
+  fh = g_new0 (FileHandle, 1);
   fh->fd = fd;
   fh->afc_cli = afc_cli;
 
-  g_free (app);
-
   g_vfs_job_open_for_write_set_handle (job, fh);
   g_vfs_job_open_for_write_set_can_seek (job, TRUE);
   g_vfs_job_open_for_write_set_can_truncate (job, TRUE);
@@ -1272,7 +1138,7 @@ g_vfs_backend_afc_replace (GVfsBackend *backend,
 {
   uint64_t fd = 0;
   GVfsBackendAfc *self;
-  char *new_path, *app = NULL;
+  char *new_path, *app;
   afc_client_t afc_cli;
   FileHandle *fh;
 
@@ -1295,7 +1161,7 @@ g_vfs_backend_afc_replace (GVfsBackend *backend,
     {
       AppInfo *info;
 
-      app = g_vfs_backend_parse_house_arrest_path (self, FALSE, filename, NULL, &new_path);
+      app = g_vfs_backend_parse_house_arrest_path (self, FALSE, filename, &new_path);
       if (app == NULL)
         {
           g_vfs_backend_afc_check (AFC_E_PERM_DENIED, G_VFS_JOB(job));
@@ -1304,11 +1170,11 @@ g_vfs_backend_afc_replace (GVfsBackend *backend,
       info = g_hash_table_lookup (self->apps, app);
       if (info == NULL)
         {
-          g_free (app);
           g_vfs_backend_afc_check (AFC_E_OBJECT_NOT_FOUND, G_VFS_JOB(job));
           return;
         }
       afc_cli = info->afc_cli;
+      g_free (app);
     }
   else
     {
@@ -1321,18 +1187,15 @@ g_vfs_backend_afc_replace (GVfsBackend *backend,
                                           G_VFS_JOB(job))))
     {
       g_free (new_path);
-      g_free (app);
       return;
     }
 
   g_free (new_path);
 
-  fh = g_vfs_backend_file_handle_new (self, app);
+  fh = g_new0 (FileHandle, 1);
   fh->fd = fd;
   fh->afc_cli = afc_cli;
 
-  g_free (app);
-
   g_vfs_job_open_for_write_set_handle (job, fh);
   g_vfs_job_open_for_write_set_can_seek (job, TRUE);
   g_vfs_job_open_for_write_set_can_truncate (job, TRUE);
@@ -1354,7 +1217,10 @@ g_vfs_backend_afc_close_read (GVfsBackend *backend,
 
   self = G_VFS_BACKEND_AFC(backend);
 
-  g_vfs_backend_file_handle_free (self, fh);
+  if (self->connected)
+    afc_file_close (fh->afc_cli, fh->fd);
+
+  g_free (fh);
 
   g_vfs_job_succeeded (G_VFS_JOB(job));
 }
@@ -1371,7 +1237,10 @@ g_vfs_backend_afc_close_write (GVfsBackend *backend,
 
   self = G_VFS_BACKEND_AFC(backend);
 
-  g_vfs_backend_file_handle_free (self, fh);
+  if (self->connected)
+    afc_file_close(fh->afc_cli, fh->fd);
+
+  g_free (fh);
 
   g_vfs_job_succeeded (G_VFS_JOB(job));
 }
@@ -1432,7 +1301,7 @@ g_vfs_backend_afc_write (GVfsBackend *backend,
   g_vfs_job_succeeded (G_VFS_JOB(job));
 }
 
-static goffset
+static int
 g_vfs_backend_afc_seek (GVfsBackendAfc *self,
                         GVfsJob *job,
                         GVfsBackendHandle handle,
@@ -1441,13 +1310,12 @@ g_vfs_backend_afc_seek (GVfsBackendAfc *self,
 {
   int afc_seek_type;
   FileHandle *fh;
-  guint64 new_offset;
 
   if ((afc_seek_type = gvfs_seek_type_to_lseek (type)) == -1)
     {
       g_vfs_job_failed(job, G_IO_ERROR, G_IO_ERROR_INVALID_ARGUMENT,
                        _("Unsupported seek type"));
-      return -1;
+      return 1;
     }
 
   fh = (FileHandle *) handle;
@@ -1455,13 +1323,11 @@ g_vfs_backend_afc_seek (GVfsBackendAfc *self,
   if (G_UNLIKELY(g_vfs_backend_afc_check (afc_file_seek (fh->afc_cli,
                                                          fh->fd, offset, afc_seek_type),
                                           job)))
-      return -1;
+    {
+      return 1;
+    }
 
-  if (G_UNLIKELY(g_vfs_backend_afc_check (afc_file_tell (fh->afc_cli, fh->fd, &new_offset),
-                                          job)))
-      return -1;
-
-  return new_offset;
+  return 0;
 }
 
 static void
@@ -1472,17 +1338,15 @@ g_vfs_backend_afc_seek_on_read (GVfsBackend *backend,
                                 GSeekType type)
 {
   GVfsBackendAfc *self;
-  goffset new_offset;
 
   g_return_if_fail (handle != NULL);
 
   self = G_VFS_BACKEND_AFC(backend);
   g_return_if_fail (self->connected);
 
-  new_offset = g_vfs_backend_afc_seek (self, G_VFS_JOB(job), handle, offset, type);
-  if (new_offset >= 0)
+  if (!g_vfs_backend_afc_seek (self, G_VFS_JOB(job), handle, offset, type))
     {
-      g_vfs_job_seek_read_set_offset (job, new_offset);
+      g_vfs_job_seek_read_set_offset (job, offset);
       g_vfs_job_succeeded (G_VFS_JOB(job));
     }
 }
@@ -1495,17 +1359,15 @@ g_vfs_backend_afc_seek_on_write (GVfsBackend *backend,
                                  GSeekType type)
 {
   GVfsBackendAfc *self;
-  goffset new_offset;
 
   g_return_if_fail (handle != NULL);
 
   self = G_VFS_BACKEND_AFC(backend);
   g_return_if_fail (self->connected);
 
-  new_offset = g_vfs_backend_afc_seek (self, G_VFS_JOB(job), handle, offset, type);
-  if (new_offset >= 0)
+  if (!g_vfs_backend_afc_seek (self, G_VFS_JOB(job), handle, offset, type))
     {
-      g_vfs_job_seek_write_set_offset (job, new_offset);
+      g_vfs_job_seek_write_set_offset (job, offset);
       g_vfs_job_succeeded (G_VFS_JOB(job));
     }
 }
@@ -1993,7 +1855,7 @@ g_vfs_backend_afc_enumerate (GVfsBackend *backend,
         }
       g_mutex_unlock (&self->apps_lock);
 
-      app = g_vfs_backend_parse_house_arrest_path (self, TRUE, path, NULL, &new_path);
+      app = g_vfs_backend_parse_house_arrest_path (self, TRUE, path, &new_path);
 
       if (app == NULL)
         {
@@ -2103,7 +1965,6 @@ g_vfs_backend_afc_query_info (GVfsBackend *backend,
   else
     {
       char *app;
-      gboolean is_doc_root;
 
       g_mutex_lock (&self->apps_lock);
       if (g_vfs_backend_load_apps (self) == FALSE)
@@ -2114,7 +1975,7 @@ g_vfs_backend_afc_query_info (GVfsBackend *backend,
         }
       g_mutex_unlock (&self->apps_lock);
 
-      app = g_vfs_backend_parse_house_arrest_path (self, TRUE, path, &is_doc_root, &new_path);
+      app = g_vfs_backend_parse_house_arrest_path (self, TRUE, path, &new_path);
 
       if (app == NULL)
         {
@@ -2134,7 +1995,7 @@ g_vfs_backend_afc_query_info (GVfsBackend *backend,
               g_vfs_backend_afc_check (AFC_E_OBJECT_NOT_FOUND, G_VFS_JOB(job));
               return;
             }
-          if (is_doc_root)
+          if (g_str_equal (new_path, "/"))
             {
               g_free (new_path);
               g_vfs_backend_afc_set_info_from_app (self, info, app_info);
@@ -2201,7 +2062,7 @@ g_vfs_backend_afc_query_fs_info (GVfsBackend *backend,
       AppInfo *info;
 
       new_path = NULL;
-      app = g_vfs_backend_parse_house_arrest_path (self, FALSE, path, NULL, &new_path);
+      app = g_vfs_backend_parse_house_arrest_path (self, FALSE, path, &new_path);
       if (app == NULL)
         {
           g_vfs_backend_afc_check (AFC_E_OP_NOT_SUPPORTED, G_VFS_JOB(job));
@@ -2258,9 +2119,6 @@ g_vfs_backend_afc_query_fs_info (GVfsBackend *backend,
   g_file_info_set_attribute_boolean (info,
                                      G_FILE_ATTRIBUTE_FILESYSTEM_READONLY,
                                      FALSE);
-  g_file_info_set_attribute_uint32 (info,
-                                    G_FILE_ATTRIBUTE_FILESYSTEM_USE_PREVIEW,
-                                    G_FILESYSTEM_PREVIEW_TYPE_IF_LOCAL);
 
   g_vfs_job_succeeded (G_VFS_JOB(job));
 }
@@ -2284,11 +2142,10 @@ g_vfs_backend_afc_set_display_name (GVfsBackend *backend,
     {
       char *app;
       AppInfo *info;
-      gboolean is_doc_root;
 
       new_path = NULL;
-      app = g_vfs_backend_parse_house_arrest_path (self, FALSE, filename, &is_doc_root, &afc_path);
-      if (app == NULL || is_doc_root)
+      app = g_vfs_backend_parse_house_arrest_path (self, FALSE, filename, &afc_path);
+      if (app == NULL || g_str_equal (afc_path, "/"))
         {
           g_free (app);
           g_free (afc_path);
@@ -2327,15 +2184,9 @@ g_vfs_backend_afc_set_display_name (GVfsBackend *backend,
       return;
     }
 
-  g_free (new_path);
-  g_free (afc_path);
-
-  /* The new path, but in the original namespace */
-  dirname = g_path_get_dirname (filename);
-  new_path = g_build_filename (dirname, display_name, NULL);
   g_vfs_job_set_display_name_set_new_path (job, new_path);
+  g_free (afc_path);
   g_free (new_path);
-  g_free (dirname);
 
   g_vfs_job_succeeded (G_VFS_JOB(job));
 }
@@ -2370,11 +2221,10 @@ g_vfs_backend_afc_set_attribute (GVfsBackend *backend,
     {
       char *app;
       AppInfo *info;
-      gboolean is_doc_root;
 
       new_path = NULL;
-      app = g_vfs_backend_parse_house_arrest_path (self, FALSE, filename, &is_doc_root, &new_path);
-      if (app == NULL || is_doc_root)
+      app = g_vfs_backend_parse_house_arrest_path (self, FALSE, filename, &new_path);
+      if (app == NULL || g_str_equal (new_path, "/"))
         {
           g_free (app);
           g_free (new_path);
@@ -2436,7 +2286,7 @@ g_vfs_backend_afc_make_directory (GVfsBackend *backend,
       AppInfo *info;
 
       new_path = NULL;
-      app = g_vfs_backend_parse_house_arrest_path (self, FALSE, path, NULL, &new_path);
+      app = g_vfs_backend_parse_house_arrest_path (self, FALSE, path, &new_path);
       if (app == NULL)
         {
           g_free (app);
@@ -2548,18 +2398,17 @@ g_vfs_backend_afc_move (GVfsBackend *backend,
     {
       AppInfo *info;
       char *app_src, *app_dst;
-      gboolean is_doc_root;
 
-      app_src = g_vfs_backend_parse_house_arrest_path (self, FALSE, source, &is_doc_root, &new_src);
-      if (app_src == NULL || is_doc_root)
+      app_src = g_vfs_backend_parse_house_arrest_path (self, FALSE, source, &new_src);
+      if (app_src == NULL || g_str_equal (new_src, "/"))
         {
           g_free (app_src);
           g_free (new_src);
           g_vfs_backend_afc_check (AFC_E_PERM_DENIED, G_VFS_JOB(job));
           return;
         }
-      app_dst = g_vfs_backend_parse_house_arrest_path (self, FALSE, destination, &is_doc_root, &new_dst);
-      if (app_dst == NULL || is_doc_root)
+      app_dst = g_vfs_backend_parse_house_arrest_path (self, FALSE, destination, &new_dst);
+      if (app_dst == NULL || g_str_equal (new_dst, "/"))
         {
           g_free (app_src);
           g_free (new_src);
@@ -2624,11 +2473,10 @@ g_vfs_backend_afc_delete (GVfsBackend *backend,
     {
       char *app;
       AppInfo *info;
-      gboolean is_doc_root;
 
       new_path = NULL;
-      app = g_vfs_backend_parse_house_arrest_path (self, FALSE, filename, &is_doc_root, &new_path);
-      if (app == NULL || is_doc_root)
+      app = g_vfs_backend_parse_house_arrest_path (self, FALSE, filename, &new_path);
+      if (app == NULL || g_str_equal (new_path, "/"))
         {
           g_free (app);
           g_free (new_path);
